import { Stack, aws_budgets as budgets } from "aws-cdk-lib";
import { IBudgetStrategyProps, IBudgetStrategy } from "./budget-strategy";
export interface IApplicationCostMonitoringProps extends IBudgetStrategyProps {
    applicationName: string;
    otherStacksIncludedInBudget?: Stack[];
}
export declare class ApplicationCostMonitoring extends IBudgetStrategy {
    readonly applicationName: string;
    private otherStacks;
    /**
     * Default Application CostMonitoring class that implements daily and monthly budgets.
     *
     * @param stack - default stack to track its resources and it will be used to define Budget resources in it.
     * @param props.applicationName - the name of application to label resources with it.
     * @param props.otherStacksIncludedInBudget - optional other stack to track their resources alog with the default stack.
     * @param props.monthlyLimitInDollars - montly limit in US Dollors.
     * @param props.defaultTopic - default SNS topic name. Only if provided, the BudgetStratgy creates an SNS topic and send notifications to it.
     * @param props.subscribers - list of email address that the CostMonitoring will use to send alerts to.
     *
     * @example tracking budget for an application called `my-application`:
     * const app = new cdk.App();
     * const firstStack = new FirstStack(app, 'FirstStack', {});
     * const secondStack = new SecondStack(app, 'SecondStack', {});
     * const costMonitoring = new ApplicationCostMonitoring(firstStack, {
     *   applicationName: 'my-application',
     *   monthlyLimitInDollars : 200,
     *  // Optional (you can add as many stack as you want)
     *   otherStacksToMonitor: [
     *     secondStack
     *   ],
     *   subscribers: [
     *     'alert@example.com'
     *   ]
     * });
     *
     * budgetStratgy.monitor();
     *
     */
    constructor(stack: Stack, props: IApplicationCostMonitoringProps);
    protected createDailyBudgets(dailyLimit: number, subscribers: Array<budgets.CfnBudget.SubscriberProperty>): void;
    protected createMonthlyBudgets(monthlyLimit: number, subscribers: Array<budgets.CfnBudget.SubscriberProperty>): void;
    protected createQuarterlyBudgets(_quarterlyLimit: number, _subscribers: budgets.CfnBudget.SubscriberProperty[]): void;
    protected createYearlyBudgets(_yearlyLimit: number, _subscribers: budgets.CfnBudget.SubscriberProperty[]): void;
    private tagAllStacks;
    /**
     * Creates all the alarms, budgets and tags all resources with the application's name.
     */
    monitor(): void;
    /**
     * Default key name for application tag.
     */
    protected get applicationTagKey(): string;
}
